#include <SoftwareSerial.h>
SoftwareSerial SerialESP8266(7,6); // RX, TX

const int flowsensor = 2;     // Flow sensor in pin 2
float kfactor = 7.5;          // K Factor: To pass from frecuency to flow
volatile int pulseCounter;    // Save number of pulses in flowsensor

const int valve = 4;          // Solenoid valve in pin 4

String server = "192.168.0.14";

//Variables to send to the webserver
int valv_state=0;
float flow=0.00;

//State variables
bool sent_valve_1 = 0;      // send to database
bool received_valve_1 = 0;  // received from database

String espString="";

// Interrupt service routine (ISR)
void ISRPulses() 
{
  pulseCounter++;         // Number of pulses +1
}

void setup() {

  //Valve is configured as OUTPUT because it will be activated through the webserver
  pinMode(valve, OUTPUT);
  digitalWrite (valve, LOW);
  
  SerialESP8266.begin(9600);
  Serial.begin(9600);
  SerialESP8266.setTimeout(2000);

  attachInterrupt(digitalPinToInterrupt(flowsensor), ISRPulses, RISING);

  //Reset the module
  SerialESP8266.println("AT+RST");
  
  //Check if ESP8266 responds
  SerialESP8266.println("AT");
  if(SerialESP8266.find("OK"))
    Serial.println("AT answer OK");
  else
    Serial.println("Error in ESP8266");

  //-----Net configuration------//

    //ESP8266 in station mode (client)
    SerialESP8266.println("AT+CWMODE=1");
    if(SerialESP8266.find("OK"))
      Serial.println("ESP8266 in station mode");
      
    //Connecting to a WIFI network
    SerialESP8266.println("AT+CWJAP=\"vodafoneDD70\",\"UFUKZTGUVNM8NE\"");   
    Serial.println("Connecting to the WIFI ...");
    SerialESP8266.setTimeout(10000); //Increase if necessary
    if(SerialESP8266.find("OK"))
      Serial.println("WIFI connected");
    else
      Serial.println("Error trying to connect to the WIFI");
    SerialESP8266.setTimeout(2000);   
    //Disable multiplex mode
    SerialESP8266.println("AT+CIPMUX=0");
    if(SerialESP8266.find("OK"))
      Serial.println("Multiplex disabled");
    
  //------End configuration-----//

  delay(1000);
  
}


void loop() {
  
    valv_state=digitalRead(valve);
    Serial.print ("Electrovalve: ");
    Serial.println (valv_state);

    noInterrupts();    // disable interrupts
    pulseCounter = 0;   //Set pulsecounter to 0 ready for calculations
    interrupts();    // re-enable interrupts

    delay (1000);   //Wait 1 second

    noInterrupts(); // disable interrupts so calculation sees an atomic value
    flow = (pulseCounter / kfactor); //Pulse frequency / K factor
    interrupts(); // re-enable interrupts.
    
    Serial.print ("Flow: ");
    Serial.print (flow, DEC); //Prints the number calculated above
    Serial.print (" l/min\r\n"); //Prints "l/min" and returns a new line
   
    //Send to php MyAdmin
    sendToDatabase();
    delay(5000);
}

void sendToDatabase()   // Conection with MySQL
{
   //Connection to the server
      
   SerialESP8266.println("AT+CIPSTART=\"TCP\",\"192.168.0.14\",80");
   if( SerialESP8266.find("OK"))
   {  
      Serial.println();
      Serial.println();
      Serial.println();
      Serial.println("ESP8266 connecting to the server...");             
              
      //Mounting the header for the http request
      String httpRequest= "GET /maintenance/getdata.php?valv_state=";
      httpRequest=httpRequest+String(valv_state)+"&flow="+String(flow)+" HTTP/1.1\r\n";
      httpRequest=httpRequest+"Host: localhost\r\n";
          
      //Send the size of the http request
      SerialESP8266.print("AT+CIPSEND=");
      SerialESP8266.println(httpRequest.length()+2);
           
      //Wait for ">" to send http request
      if(SerialESP8266.find(">"))
      {
         Serial.println("Sending HTTP request ...");
         SerialESP8266.println(httpRequest);
         if( SerialESP8266.find("SEND OK"))
         {  
           Serial.println("HTTP request sent:");
           Serial.println();
           Serial.println(httpRequest);
            
           boolean endResponse=false; 
           long initialTime=millis(); 
           espString="";
              
           while(endResponse==false)
           {
              while(SerialESP8266.available()>0) 
              {
                 char c=SerialESP8266.read();
                 //Serial.write(c);
                 espString.concat(c);  //Save the response in the string "espString"
              }
              if(espString.indexOf("Warning")>0) //If a Warning returns, means database is not connected
              {
                 Serial.println();
                 Serial.println("Cannot connect to MySQL database");         
                 endResponse=true;
              }              
              if((millis()-initialTime)>10000) //Finishing if 10 secs have passed
              {
                 Serial.println("Timeout");
                 SerialESP8266.println("AT+CIPCLOSE");
                 if( SerialESP8266.find("OK"))
                   Serial.println("Connection finished");
                 endResponse=true;
              }
              if(espString.indexOf("_b1")>0) //Locate b1, if we receive it, response has finished.
              {
                 String state_text = "";
                 int strposition = espString.indexOf("_b1");
                 int chg_state = espString.substring(strposition+3,strposition+4).toInt();
                 if (chg_state == 0){
                    state_text = "LOW";
                    digitalWrite (valve, LOW);
                 }else if (chg_state == 1){
                    state_text = "HIGH";                  
                    digitalWrite (valve, HIGH);
                 }
                 Serial.println();
                 Serial.print("Changed electrovalve state to " + state_text);
                 Serial.println();                 
                 endResponse=true;                 
              }              
           }             
         }
         else
         {
            Serial.println("It's not possible to send http request...");
         }            
      }
   }
   else
   {
      Serial.println("It's not possible the connection with the server");
   }
}
